# Generated by roxygen2: do not edit by hand

export(EMforCFR)
export(dic.fit)
export(dic.fit.mcmc)
export(generate.coarse.data)
export(get.obs.type)
export(loglikhd)
export(precision.simulation)
export(precision.simulation.coarse)
export(precision.simulation.exact)
exportClasses(cd.fit)
exportClasses(cd.fit.mcmc)
exportMethods(logLik)
importFrom(MCMCpack,MCMCmetrop1R)
importFrom(grDevices,rgb)
importFrom(graphics,lines)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,coef)
importFrom(stats,dgamma)
importFrom(stats,dlnorm)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dweibull)
importFrom(stats,glm)
importFrom(stats,integrate)
importFrom(stats,logLik)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pgamma)
importFrom(stats,plnorm)
importFrom(stats,poisson)
importFrom(stats,pweibull)
importFrom(stats,qgamma)
importFrom(stats,qlnorm)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,qweibull)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,vcov)
importFrom(utils,data)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
